function [m,Q,C,R] = fa_learn(y, m,Q,C,R, errThd, Temp, wY)
%
% fa_learn -- learning of factor analysis parameters
%
% Usage:
%   [m,Q,C,R] = fa_learn(y, m,Q,C,R, errThd, Temp, wY)
%
% Input:
%   y		matrix of data vectors
%   m		initial state offset
%   Q		initial state noise variance
%   C		initial observation matrix
%   R		initial observation noise variance
%   errThd	learning error threshold
%   Temp	annealing temperature (default 1)
%   wY		distribution of observations (default: uninformative)
%
% Output:
%   m		reestimated state offset
%   Q		reestimated state noise variance
%   C		reestimated observation matrix
%   R		reestimated observation noise variance
%
% Global variables:
%   FA_NL_C	set to disable reestimation of C (default empty)
%   FA_NL_Q	set to disable reestimation of Q (default empty)
%   FA_NL_R	set to disable reestimation of R (default empty)
%   FA_NL_m	set to disable reestimation of m (default empty)
%

%	$Revision: 2.0 $ $Date: 2000/04/24 17:42:40 $

global FA_VERBOSE FA_MAX_FUN
global FA_KMAX_LEARN

if isempty( FA_KMAX_LEARN )
  kMax = 100;
else
  kMax = FA_KMAX_LEARN;
end

N     = size(Q,1);
[M,L] = size(y);

if (nargin<7)
  Temp = 1;
end

if (nargin<8)
  wY = ones(1,L)/L;
end

cost = realmax;
err  = realmax;

k = 0;

while( k<kMax & abs(err)>errThd )
   
   % Initialize stats
   [Ex,Exx,Eyx,Eyy] = fa_ics(N,M);
   
   % Expectation / inference
   % Find Q(x) = P(x|y), or Q(x) \approx P(x|y)
   % Actually, find Q such that it minimizes costE,
   % costE = E[ -log P(y,x|old_params) ]_Q(x) - E[ -log Q(y,x) ]_Q(x)
   %         - log Q(y)
   [costEs,ex,vx] = fa_infr(y, m,Q,C,R*Temp);
   
   costE = wY*costEs';
   
   % Update statistics
   [Ex,Exx,Eyx,Eyy] = fa_ucs(y,ex,vx, Ex,Exx,Eyx,Eyy,wY, m,Q,C,R);
   
   % Cost before M, costB = E[ -log P(y,x|old_params) ]_Q(x)
   costB = fa_cost(m,Q,C,R, Ex,Exx,Eyx,Eyy);
     
   % Maximization
   if isempty(FA_MAX_FUN)
     [m,Q,C,R] = fa_max(Ex,Exx,Eyx,Eyy, m,Q,C,R);
   else
     [m,Q,C,R] = feval( FA_MAX_FUN, Ex,Exx,Eyx,Eyy, m,Q,C,R );
   end
   
   % Cost after M,  costA = E[ -log P(y,x|new_params) ]_Q(x)
   costA = fa_cost(m,Q,C,R, Ex,Exx,Eyx,Eyy);
   
   % Total cost after minimization, E[ -log P(y,x|new_params) ]_Q(x) - E
   % [ -log Q(y,x) ]_Q(x) - log Q(y)
   cost(end+1) = costE + costA - costB;
   % cost(end+1) = costE;
   
   % Relative error
   err = ( cost(end-1)-cost(end) ) / abs( cost(end) );
   
   if ~isempty( FA_VERBOSE )
     if (Temp>1)
       fprintf('Iter (learning): %3d, Temp = %8.2e, Cost: %8.4f , Err: %.4f%%\n'...
	       ,k, Temp, cost(end),err*100);
     else
       fprintf('Iter (learning): %3d, Cost: %8.4f , Err: %.4f%%\n', k, ...
	       cost(end),err*100);
     end
   end
   
   %%%%%%%%%%%
   % Annealing
   %%%%%%%%%%%
   Temp = (Temp + 1)/2;
   
   k = k + 1;
   
end

