function [cost,es,cs,scale] = dhmm_fwd(y,Pss,Ps0,Q,wS,wY)
%
% dhmm_fwd -- Discrete HMM forward state estimation 
%             (forward probability propagation)
%
% Usage:
%   [cost,es,cs,scale] = dhmm_fwd(y,Pss,Ps0,Q,wS,wY)
%
% Input:
%   y		observation sequence [ 1 x time ]
%   Pss		state transition probabilities [ to x from ]
%   Ps0		initial state probabilities [ state x 1 ]
%   Q		observation  probabilities  [ obs x state ]
%   wS		state transition probability weights (default 1)
%   wY		observation  probability weights (default 1)
%
% Output:
%   cost	-log Pr(Y_T) / T  
%   es		E[ s_t | Y_t ] = Pr( s_t | observation sequence up to time 't')
%   cs		E[ s_t s_{t-1}' | Y_t]
%   scale	log Pr( s_0, ..., s_t | observation sequence up to time 't')
%
% Note:
%   See notes for 'hmm_vite'.
%
% See also:
%   dhmm, dhmm_bwd, dhmm_vite, dhmm_smt, dhmm_infr
%

%	$Revision: 2.0 $ $Date: 2000/04/24 17:43:53 $

N = size(Pss,1);
i = ones(1,N);
T = size(y,2);

if (nargin<6)
  wS = ones(1,T);
end

if (nargin<6)
  wY = ones(1,T);
end

% Forward
Pys = (Q(y(1),:)').^wY(1);
es0 = Ps0.*Pys;
scale(1) = sum(es0);
es(:,1) = es0 / scale(1);

for t=2:T,
  Pys = ( Q(y(t),:)' ) .^ wY(t);
  es0 = ( ( Pss.^wS(t) ) * es(:,t-1)).*Pys;
  scale(t) = sum(es0);
  es(:,t) = es0 / scale(t);
  if (nargout>2)
    cs(:,:,t-1) = ( Pss.^wS(t) ) .* es(:,(t-1)*i)' .* Pys(:,i) / scale(t);
  end
end

scale = log(scale);
cost  = -sum(scale)/T;
