function [cost,es,cs] = dhmm_infr(y,Pss,ps0,Q,wS,wY)
%
% dmm_infr -- discrete HMM inference
%
% Usage:
%   [cost,es,cs] = dmm_infr(y,Pss,ps0,Q,wS,wY)
%
% Input:
%   y		observation sequence
%   Pss		state transition probabilities [ to x from ]
%   Ps0		initial state probabilities [ state x 1 ]
%   Q		observation probabilities [ obs x state ]
%   wS		state transition probability weights (default 1)
%   wY		observation probability weights (default 1)
%
% Output:
%   cost	log Pr( observation sequence )
%   es		E[ s_t | observation sequence ]
%   cs		E[ s_t s_{t-1}' | observation sequence ]
%
% Global settings:
%   INFERENCE_MODE_HMM	'exact'	-- exact inference (default)
%			'vite'	-- Viterbi (winner-takes-all) inference
%
% See also:
%   hmm_smt, hmm_vite, hmm_fwd, hmm_bwd, hmm_sim
%

%	$Revision: 2.0 $ $Date: 2000/04/24 17:43:54 $

global DHMM_INFERENCE_MODE
if (~isempty(DHMM_INFERENCE_MODE))
  inferMode = DHMM_INFERENCE_MODE;
else
  inferMode = 'exact';
end

T = size(y,2);
[M,N] = size(Q);

if (nargin<5)
  wS = ones(1,T);
end

if (nargin<6)
  wY = ones(1,T);
end

switch(lower(inferMode))
  
case 'exact',
  
  % Do regular inference
  
  switch (nargout),
  case 1,
    cost = dhmm_smt(y,Pss,ps0,Q,wS,wY);
  case 2,
    [cost,es] = dhmm_smt(y,Pss,ps0,Q,wS,wY);
  otherwise,
    [cost,es,cs] = dhmm_smt(y,Pss,ps0,Q,wS,wY);
  end

case 'vite',

  
  % Do Viterbi inference
  [cost,is] = dhmm_vite(y,Pss,ps0,Q,wS,wY);
  
  if (nargout>1)
    % Find es from is
    base = eye(N);
    es = base(:,is);
  end
  
  if (nargout>2)
  % Find cs from es
    for t=1:T-1,
      cs(:,:,t) = es(:,t+1)*es(:,t)';
    end
  end

 otherwise,
  
  error( sprintf('Unknown DHMM_INFERENCE_MODE option %s.', upper(inferMode)) );
  
end


