function [P,Q,p0,es,cs,ec] = dhmm_init( y, N, M, pY, mode, varargin)
%
% dhmm_init -- initialize discrete observation HMM parameters
%
% Usage:
%   [P,Q,p0,es,cs,ec] = dhmm_init( y, N, M, pY, pmode, ...)
%
% Input:
%   y		list of observation sequences
%   N		number of states
%   M		number of observations
%   pY		distribution of observation sequences
%   pmode	state transition matrix mode: see 'mc_gen' mode
%
% Output:
%   P		state transition probability matrix
%   Q		mixture weights
%   p0		initial state distribution
%   es		E[ s_t | observations ]
%   cs		E[ s_t s_{t-1}' | observations ]
%   ec		E[ c_t s_t | observations ]
% 
%

%	$Revision: 2.0 $ $Date: 2000/04/24 17:43:54 $

if (~iscell(y))
  y = {y};
end

% Number of sequences
L = length(y);

if nargin<4 | isempty(pY),
  pY = ones(1,L)/L;
end

% Basis vectors
basisS = eye(N);
basisY = eye(M);

% Initialize scores
[Eys,Ess,Es0,T] = dhmm_ics(N,M);

for m=1:L,
   
  Tm = size(y{m},2);
  
  if (~strcmp(mode,'full'))
    es{m} = mc_gen( N, Tm, mode, varargin );
  else
    [tmp,tmp,tmp,tmp,tmp,map] = vq_flat( basisY(:,y{m}), N );
    es{m} = basisS(:,map);
  end
  
  for t=1:Tm-1,
    
    cs{m}(:,:,t) = es{m}(:,t+1)*es{m}(:,t)';
    
  end

  % Update scores
  [Eys,Ess,Es0,T] = dhmm_ucs(y{m},es{m},cs{m}, Eys,Ess,Es0,T, [],[],[], pY(m));
  
end % m=1:L

% Maximize
[P,Q,p0] = dhmm_max(Eys,Ess,Es0,T);
