function [P,Q,p0, es,cost] =  dhmm_learn(y, P,Q,p0, errThd,wY)
%
% dhmm_learn -- EM parameter estimation for discrete observation
%               HMMs 
%
% Usage:
%   [P,Q,p0, es,is] =  dhmm_learn(y, P,Q,p0, err,w)
%
% Input:
%   y	list of observation sequences
%   P	initial state transition probability matrix
%   Q	initial observation probability matrix
%   p0	initial state probabilities
%   err	max. relative error
%   w	distribution of observation sequences (default uninformative)
%
% Output:
%   P	updated state transition probability matrix
%   Q	updated observation probability matrix
%   p0	updated initial state probabilities
%   es	list of E[ s_t | observation sequence]
%   is	list of Viterbi estimates of state sequences
%
% Global variables:
%   DHMM_VERBOSE	print information during iterations (default [])
%   DHMM_KMAX_LEARN	maximal number of iterations allowed (default 30)
%   DHMM_MAX_FUN	dhmm parameter maximization function (default dhmm_max)
%
% Note:
%   (1) List of observation sequences 'y' and state estimates 'es' and 'is'
%   are MATLAB cell arrays.
%   (2) Expectation (E) step can be exact or approximate. See 'hmm_infr'.
%
% See also:
%   hmm, hmm_sim, hmm_infr, hmm_vite, hmm_smt, mog
%

%	$Revision: 2.0 $ $Date: 2000/04/24 17:43:54 $

global DHMM_VERBOSE DHMM_MAX_FUN
global DHMM_KMAX_LEARN

cost = realmax;
err  = realmax;

if (~isempty(DHMM_KMAX_LEARN))
   kMax = DHMM_KMAX_LEARN;
else
   kMax = 30;
end

% Param dimensions
N = size(P,1);
M = size(Q,1);

% Multiplicity of data sequences
if (~iscell(y))
   y = {y};
end
S = length(y);

if nargin<6,
  wY = ones(1,S)/S;
end

% fprintf('Processing %d data sequences...\n', S);

k = 0;
while(k < kMax & abs(err)>errThd)
  
  % Initialize scores
  [Eys,Ess,Es0,T] = dhmm_ics(N,M);
  
  costE = 0;
  % For each data sequence
  for i=1:S,
    
    Y = y{i};
    Ti = size(Y,2);
    
    if (~isempty(DHMM_VERBOSE))
      % fprintf('Data sequence %d ( %d samples ).\n', i,Ti);
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%
    % Expectation (inference)
    %%%%%%%%%%%%%%%%%%%%%%%%%
    [costEi,es{i},cs] = dhmm_infr( Y, P, p0, Q );
    
    costE = costE + Ti*wY(i)*costEi;
    
    % Update cumulative scores
    [Eys,Ess,Es0,T] = dhmm_ucs( Y,es{i},cs, Eys,Ess,Es0,T, ...
				    P,Q,p0, wY(i) );
    
  end % for i=1:S
  
  % Cost before maximization
  costB = dhmm_cost(P,Q,p0, Eys,Ess,Es0,T);
  
  %%%%%%%%%%%%%%
  % Maximization
  %%%%%%%%%%%%%%
  if isempty(DHMM_MAX_FUN)
    [P,Q,p0] = dhmm_max(Eys,Ess,Es0,T, P,Q,p0);
  else
    [P,Q,p0] = feval( DHMM_MAX_FUN, Eys,Ess,Es0,T, P,Q,p0 );
  end    
  
  % Cost after maximization
  costA = dhmm_cost(P,Q,p0, Eys,Ess,Es0,T);
  
  % Actual cost
  cost(end+1) = (costE + costA - costB)/T;
  
  % Relative error
  err = ( cost(end-1) - cost(end) ) / abs( cost(end) );
  
  if (~isempty(DHMM_VERBOSE))
    fprintf('Iter (learning): %3d, Cost = %f, Err = %.4f%%\n', k, ...
	    cost(end), err*100);
  end
   
  k = k + 1;
  
  
end % while

