function [cost,es,cs,scale] = ghmm_fwd(y,Pss,Ps0,Q,C,R,wS,wY)
%
% hmm_fwd -- GHMM forward state estimation (forward probability propagation)
%
% Usage:
%   [cost,es,cs,scale] = ghmm_fwd(y,Pss,Ps0,Q,C,R,wS,wY)
%
% Input:
%   y		observation vector sequence [ dim x time ]
%   Pss		state transition probabilities [ to x from ]
%   Ps0		initial state probabilities [ state x 1 ]
%   Q		mixture weights [ mixture x state ]
%   C		observation mean [ dim x mixture x state ]
%   R		observation variance: diagonal [ dim x mixture x state ]
%		                      full [ dim x dim x mixture x state ]
%   wS		state transition probability weights (default 1)
%   wY		observation probability weights (default 1)
%
% Output:
%   cost	-log Pr(Y_T) / T  
%   es		E[ s_t | Y_t ] = Pr( s_t | observation sequence up to time 't')
%   cs		E[ s_t s_{t-1}' | Y_t]
%   scale	log Pr( s_0, ..., s_t | observation sequence up to time 't')
%
% Note:
%   See notes for 'hmm_vite'.
%
% See also:
%   ghmm, mog, ghmm_bwd, ghmm_vite, ghmm_smt, ghmm_infr
%

%	$Revision: 2.0 $ $Date: 2000/04/24 17:43:57 $

N = size(Pss,1);
i = ones(1,N);
T = size(y,2);

if (nargin<7)
  wS = ones(1,T);
end

if (nargin<8)
  wY = ones(1,T);
end

% Forward
Pys = exp( wY(1)*mog_eval(y(:,1),Q,C,R) );
es0 = Ps0.*Pys;
scale(1) = sum(es0);
es(:,1) = es0 / scale(1);

for t=2:T,
  Pys = exp( wY(t)*mog_eval(y(:,t),Q,C,R) );
  es0 = ( ( Pss.^wS(t) ) * es(:,t-1)).*Pys;
  scale(t) = sum(es0);
  es(:,t) = es0 / scale(t);
  if (nargout>2)
    cs(:,:,t-1) = ( Pss.^wS(t) ) .* es(:,(t-1)*i)' .* Pys(:,i) / scale(t);
  end
end

scale = log(scale);
