function [cost,es,cs,ec] = ghmm_infr(y, P,Q,C,R,p0,wS,wY);
%
% ghmm_infr -- state inference in Gaussian mixture observation HMMs
%
% Usage:
%   [cost,es,cs,ec] = ghmm_infr(y, P,Q,C,R,p0, wS,wY);
%
% Input:
%   y		observation sequence [ DIM x time ]
%   P		state transition log probability matrix [ to x from ]
%   Q		mixture weights [ mixture x state ]
%   C		observation means [ DIM x mixture x state ]
%   R		observation variances:	diagonal [ DIM x mixture x state ] or 
%					full [ DIM x DIM x mixture x state ]
%   p0		initial state probabilities
%   wS		state transition probability weights (default 1)
%   wY		observation probability weights (default 1)
%
% Output:
%   cost	log Pr( observation sequence )
%   es		E[ s_t | observation sequence ]
%   cs		E[ s_t s_{t-1}' | observation sequence ]
%
% Global variables:
%   INFERENCE_MODE_HMM	'smooth' -- exact inference (default)
%			'vite'   -- Viterbi inference
% See also:
%   ghmm, mog, ghmm_smt, ghmm_vite
%

%	$Revision: 2.0 $ $Date: 2000/04/24 17:43:57 $


global GHMM_INFERENCE_MODE
if (~isempty(GHMM_INFERENCE_MODE))
  inferMode = GHMM_INFERENCE_MODE;
else
  inferMode = 'exact';
end

T = size(y,2);
[M,N] = size(Q);

if nargin<7,
  wS = ones(1,T);
end

if nargin<8,
  wY = ones(1,T);
end

switch(lower(inferMode))
  
case 'exact',
  
  % Do regular inference
  
  switch (nargout),
  case 1,
    cost = ghmm_smt(y,P,p0,Q,C,R,wS,wY);
  case 2,
    [cost,es] = ghmm_smt(y,P,p0,Q,C,R,wS,wY);
  otherwise,
    [cost,es,cs] = ghmm_smt(y,P,p0,Q,C,R,wS,wY);
  end
  
case 'vite',

  
  % Do Viterbi inference
  [cost,is] = ghmm_vite(y,P,p0,Q,C,R,wS,wY);
  
  if (nargout>1)
    % Find es from is
    base = eye(N);
    es = base(:,is);
  end

  
  if (nargout>2)
  % Find cs from es
    for t=1:T-1,
      cs(:,:,t) = es(:,t+1)*es(:,t)';
    end
  end
    
 otherwise,
  
  error( sprintf('Unknown GHMM_INFERENCE_MODE option %s.', upper(inferMode)) );
  
end


if (nargout>3)
  
  [Pys,Pycs] = mog_eval( y, Q, C, R );
  
  ec = zeros(M,N,T);
  
  tmp = permute( Pys, [ 3 1 2 ]); tmp = tmp(ones(M,1),:,:);
  nz  = find( ~isinf( tmp ) );
  ec(nz) = exp( Pycs(nz) - tmp(nz) );
  
end
