function [P,Q,C,R,p0,es,cs,ec] = ghmm_init( y, N, M, d, pY, mode, varargin)
%
% ghmm_init -- initialize GHMM parameters
%
% Usage:
%   [P,Q,C,R,p0,es,cs,ec] = ghmm_init( y, N, M, vmode, pY, pmode, ...)
%
% Input:
%   y		list of observation sequences
%   N		number of states
%   M		number of mixture components per state
%   vmode	Gaussian mixture mode: 0 - diagonal, 1 - full
%   pY		distribution of observation sequences
%   pmode	state transition matrix mode: see 'mc_gen' mode
%
% Output:
%   P		state transition probability matrix
%   Q		mixture weights
%   C		mixture means
%   R		mixture variances
%   es		E[ s_t | observations ]
%   cs		E[ s_t s_{t-1}' | observations ]
%   ec		E[ c_t s_t | observations ]
% 
% Global variables:
%

%	$Revision: 2.0 $ $Date: 2000/04/24 17:43:57 $

if (~iscell(y))
  y = {y};
end

% Number of sequences
L = length(y);
D = size(y{1},1);

if nargin<5 | isempty(pY),
  pY = ones(1,L)/L;
end

% Basis vectors for mixtures
baseN = eye(N);
baseM = eye(M);

% Initialize scores
[Ess,Ecs,Eycs,Eyycs,Es0,Dss,Ds,T] = ghmm_ics(N,M,D,d);

for m=1:L,
   
  Tm = size(y{m},2);
  
  % Number of states to "work on"
  Ns   = N;
  Noff = 0;
  
  % Initialize state segmentation
  if strcmp(mode,'full')
    [tmp,tmp,tmp,tmp,tmp,map] = vq_flat( y{m}, Ns );
    es{m} = baseN(:,map+Noff);
  elseif strcmp(mode,'null')
    es{m} = varargin{1}{m};
  else
    es{m} = mc_gen( Ns, Tm, mode, varargin );
  end
  
  % Initialize MOG weights using VQ
  ec{m} = zeros(M,N,Tm);
  
  for n = 1:N,
    
    t = find(es{m}(n,:));
        
    if (~isempty(t)),
      
      if (M>1)
	[tmp,tmp,tmp,tmp,tmp,map] = vq_flat( y{m}(:,t), M );
	ec{m}(1:M,n,t) = baseM(:,map);
      else
	ec{m}(1:M,n,t) = 1;
      end
      
      
    end
    
  end
      
  for t=1:Tm-1,
    
    cs{m}(:,:,t) = es{m}(:,t+1)*es{m}(:,t)';
    
  end

  % Update scores
  [Ess,Ecs,Eycs,Eyycs,Es0,Dss,Ds,T] = ghmm_ucs(y{m},es{m},cs{m},ec{m}, ...
      Ess,Ecs,Eycs,Eyycs,Es0,Dss,Ds,T,pY(m));
  
end % m=1:L

% Maximize
[P,Q,C,R,p0] = ghmm_max(Ess,Ecs,Eycs,Eyycs,Es0,Dss,Ds,T,L);
