function [P,Q,C,R,p0,cost,es] =  ghmm_learn(y, P,Q,C,R,p0, errThd, wY)
%
% ghmm_learn -- EM parameter estimation for Gaussian mixture observation
%               HMMs 
%
% Usage:
%   [P,Q,C,R,p0, cost,es] =  ghmm_learn(y, P,Q,C,R,p0, err, wY)
%
% Input:
%   y	list of observation sequences
%   P	initial state transition probability matrix
%   Q	initial mixture weights
%   C	initial observation means
%   R	initial observation variances
%   p0	initial state probabilities
%   err	max. relative error
%   wY	distribution of observations (default: uninformative)
%
% Output:
%   P	updated state transition probability matrix
%   Q	updated mixture weights
%   C	updated observation means
%   R	updated observation variances
%   p0	updated initial state probabilities
%   es	list of E[ s_t | observation sequence]
%   is	list of Viterbi estimates of state sequences
%
% Global variables:
%   GHMM_VERBOSE	print information during iterations (default [])
%   GHMM_KMAX_LEARN	maximal number of iterations allowed (default 30)
%   GHMM_MAX_FUN	ghmm parameter maximization function (default ghmm_max)
%
% Note:
%   (1) List of observation sequences 'y' and state estimates 'es' and 'is'
%   are MATLAB cell arrays.
%   (2) Expectation (E) step can be exact or approximate. See 'ghmm_infr'.
%
% See also:
%   ghmm, ghmm_sim, ghmm_infr, ghmm_vite, ghmm_smt, mog
%

%	$Revision: 2.0 $ $Date: 2000/04/24 17:43:58 $

global GHMM_VERBOSE GHMM_MAX_FUN
global GHMM_KMAX_LEARN

cost = realmax;
err  = realmax;

if (~isempty(GHMM_KMAX_LEARN))
   kMax = GHMM_KMAX_LEARN;
else
   kMax = 30;
end

% Param dimensions
D = size(C,1);
N = size(P,1);
M = size(Q,1);
d = ndims(R)-ndims(C); % determines "diagonality" of R

% Multiplicity of data sequences
if (~iscell(y))
   y = {y};
end
S = length(y);

if nargin<8,
  wY = ones(1,S)/S;
end

% fprintf('Processing %d data sequences...\n', S);

k = 0;
while(k < kMax & abs(err)>errThd)
  
  % Initialize scores
  [Ess,Ecs,Eycs,Eyycs,Es0,Dss,Ds,T] = ghmm_ics(N,M,D,d);
  
  costE = 0;
  % For each data sequence
  for i=1:S,
    
    Y = y{i};
    Ti = size(Y,2);
    
    if (~isempty(GHMM_VERBOSE))
      % fprintf('Data sequence %d ( %d samples ).\n', i,Ti);
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%
    % Expectation (inference)
    %%%%%%%%%%%%%%%%%%%%%%%%%
    [costEi,es{i},cs,ec] = ghmm_infr(Y, P,Q,C,R,p0);
    
    costE = costE + costEi*Ti*wY(i);
    
    % Update cumulative scores
    [Ess,Ecs,Eycs,Eyycs,Es0,Dss,Ds,T] = ghmm_ucs(Y,es{i},cs,ec, ...
	Ess,Ecs,Eycs,Eyycs,Es0,Dss,Ds,T,wY(i));
    
  end % for i=1:S
  
  % Cost before maximization
  costB = ghmm_cost(P,Q,C,R,p0, Ecs,Ess,Es0,Dss,Eyycs,Eycs,T);
  
  %%%%%%%%%%%%%%
  % Maximization
  %%%%%%%%%%%%%%
  if isempty(GHMM_MAX_FUN)
    [P,Q,C,R,p0] = ghmm_max(Ess,Ecs,Eycs,Eyycs,Es0,Dss,Ds,T,P,Q,C,R,p0);
  else
    [P,Q,C,R,p0] = feval( GHMM_MAX_FUN, Ess,Ecs,Eycs,Eyycs,Es0,Dss,Ds,T, ...
			  P,Q,C,R,p0 );
  end    
    
  % Cost after maximization
  costA = ghmm_cost(P,Q,C,R,p0, Ecs,Ess,Es0,Dss,Eyycs,Eycs,T);
  
  % Actual cost
  cost(end+1) = (costE + costA - costB)/T;
  
  % Relative error
  err = ( cost(end-1) - cost(end) ) / abs( cost(end) );
  
  if (~isempty(GHMM_VERBOSE))
    fprintf('Iter (learning): %3d, Cost = %f, Err = %.4f%%\n', k, ...
	    cost(end), err*100);
  end
   
  k = k + 1;
  
  
end % while

