function [y,s,c] = ghmm_sim(P,Q,C,R,p0,T,seed)
%
% ghmm_sim -- simulate Gaussian mixture HMM
%
% Usage:
%   [y,s,c] = ghmm_sim(Pss,Q,C,R,Ps0,T,seed)
%
% Input:
%   Pss		state transition probabilities [ to x from ]
%   Q		mixture weights [ mixture x state ]
%   C		observation mean [ dim x mixture x state ]
%   R		observation variance: diagonal [ dim x mixture x state ]
%		                      full [ dim x dim x mixture x state ]
%   Ps0		initial state log probabilities [ state x 1 ]
%   T		duration of simulation
%   seed	random generator seed
%
% Output:
%   y		observation sequences
%   x		hidden state sequence
%   c		mixture component sequence
%
% See also:
%   ghmm, mog
%

%	$Revision: 2.1 $ $Date: 2000/07/12 18:11:04 $

if (nargin<7), seed = 0; end
if (nargin<8), mode = 'exp'; end
if (nargin<9), param = []; end

% Diagonal covariance mode flag
diagMode = (ndims(R)==ndims(C));

rand('seed',seed);
randn('seed',seed);

N = size(P,1);
M = size(Q,1);
Na = size(C,1);

% Cumulative sums
cPss = cumsum(P,1);
cPcs = cumsum(Q,1);
cPs0 = cumsum(p0);

% Decompose R
if (~diagMode)
  for s = 1:N,
    for m = 1:M,
      [Ur(:,:,m,s),DIAGr(:,m,s),tmp] = svd(R(:,:,m,s));
    end
  end
end

% Simulate Markov chain
s = mc_sim(P,p0,T,seed);

% Simulate observations
for t=1:length(s),

  rC = rand(1);
  rY = randn(Na,1);

  c(t) = sum( rC > cPcs(:,s(t)) , 1 ) + 1;
  
  if (c(t)>M),
    s = s(:,1:end-1);
    c = c(:,1:end-1);
    break;
  end
  
  if (diagMode)
    y(:,t) = C(:,c(t),s(t)) + sqrt( R(:,c(t),s(t)) ).* rY;
  else
    y(:,t) = C(:,c(t),s(t)) + ...
	Ur(:,:,c(t),s(t))*sqrt( DIAGr(:,c(t),s(t)) ).* rY;
  end
  
end
    

