% HMM demo script

%	$Revision: 2.0 $ $Date: 2000/04/24 17:43:59 $

clear all
warning off
echo on
clf

% **********************************************************************
% Discrete observation HMM with three hidden states and five observation
% symbols
% **********************************************************************

% This is an ergodic HMM
Pss = [ .9	0	.1 ;
	.1	.85	0  ;
	0	.15	.9 ];
% i.e., Pr( s_t = 3 | s_{t-1} = 2) = .15


% Initial state is state '1'
Ps0 = [ 1 ; 0 ; 0 ];

% Observation probabilities
Py  = [ .8	0	0  ;
	.1	.05	0  ;
	.05	.15	.05;
	.05	.75	.1 ;
	0	.05	.85 ];
% i.e., Pr( y_t = 4 | s_t = 2 ) = .75
pause

% Simulate S=5 sequences of length T=100 using these parameters
S = 5;
T = 100;

% Assume some arbitrary distribution of sequences
pY = rand(1,S); pY = pY/sum(pY);

% ...then simulate the model using default random seed of '0'
for i=1:S,
  [y{i},s{i}] = dhmm_sim(Pss,Py,Ps0,T,rand(1)*1e3);
end

% plot the hidden states 's' and observations 'y'
subplot(311); h(1,1) = stairs(s{1}); title('Hidden states - s');
subplot(312); stairs(y{1}); title('Observations - y');

pause

% Now find estimates (probabilities) of hidden states given sequence 'y'
% using HMM inference

% To do that one first needs to evaluate Pys = Pr(y_t|s_t) for all s_t
Pys = Py(y{1},:)';

% Then call hmm_smt
[cost,es] = dhmm_smt( y{1}, Pss, Ps0, Py  );

% Alternatively, one could first evaluate Pys = Pr(y_t|s_t) for all s_t
Pys = Py(y{1},:)';

% and the call hmm_smt
[cost,es_alt] = hmm_smt( Pss, log(Pys), Ps0 );

% Obviously, there is no difference
norm(es-es_alt)

% plot probabilities of hidden states as a function of time
subplot(313); plot(es'); title('Probabilities of hidden states');
legend('Pr(s_t=1)','Pr(s_t=2)','Pr(s_t=3)');

% also plot it in index space and compare to ground trooth
subplot(311); hold on; h(1,2) = plot( ([1 2 3]*es),'r' );

% That should look similar to what we know generated the observations,
% i.e., sequence 's'
basis = eye(3);
subplot(313); hold on; plot(basis(:,s{1})',':'); hold off

pause

% Another possibility is to try Viterbi decoding to find the 'best' sequence of
% hidden states
[score,idx] = dhmm_vite( y{1}, Pss, Ps0, Py  );

% Or, alternatively, 
[score_alt,idx_alt] = hmm_vite( Pss, log(Pys), Ps0 );

% plot it, compared to 's'
subplot(311); hold on; h(1,3) = stairs(idx,'k'); legend(h(1,:),'s','soft','Viterbi'); hold off

pause

% One can also estimate parameters of this HMM given the data
% sequence 'y'

% Enable printing of scores
global DHMM_VERBOSE DHMM_INFERENCE_MOD
DHMM_VERBOSE = 1;

% Initialize parameters using VQ
[Pss_0,Py_0,Ps0_0] = dhmm_init( y, 3, 5, pY, 'full');

% Now apply EM learning using exact inference
[Pss_1,Py_1,Ps0_1] = dhmm_learn(y, Pss_0,Py_0,Ps0_0, 1e-4,pY);

printmat(Pss); printmat(Pss_0); printmat(Pss_1);
pause

printmat(Py); printmat(Py_0); printmat(Py_1);
pause


clf
% **********************************************************************
% Gaussian observation HMM with three hidden states and five observation
% symbols
% **********************************************************************

% This is a letf-to-right HMM
Pss = [ .95	0	0 ;
	.05	.98	0  ;
	0	.02	1 ];
% i.e., Pr( s_t = 3 | s_{t-1} = 2) = .02

% Initial state is state '1'
Ps0 = [ 1 ; 0 ; 0 ];

% Two mixture components per state.  Mixing probabilities are
Q = [ .8 .7 .4 ;
      .2 .3 .6 ];

% Observation means for states '1' through '3'
C(:,:,1) = [ 0 5 ]; % i.e., state '1' mixture '1' has mean '0'
C(:,:,2) = [ 10 12 ];
C(:,:,3) = [ 20 26 ];

% Observation variances
R(:,:,1) = [ 1 1 ];
R(:,:,2) = [ 2 1 ];
R(:,:,3) = [ 2 2 ];

pause

% Simulate a sequence of observations
[y,s,c] = ghmm_sim(Pss,Q,C,R,Ps0,T, 12345);

% plot the hidden states 's' and observations 'y'
subplot(311); h(1) = stairs(s); title('Hidden states - s');
subplot(312); plot(y); title('Observations - y');

pause

% Now find estimates (probabilities) of hidden states given sequence 'y'
% using HMM inference
[cost,es] = ghmm_smt(y, Pss, Ps0, Q, C, R);

% plot probabilities of hidden states as a function of time
subplot(313); plot(es'); title('Probabilities of hidden states');
legend('Pr(s_t=1)','Pr(s_t=2)','Pr(s_t=3)');

% also plot it in index space and compare to ground trooth
subplot(311); hold on; h(2) = plot( ([1 2 3]*es),'r' );

% That should look similar to what we know generated the observations,
% i.e., sequence 's'
basis = eye(3);
subplot(313); hold on; plot(basis(:,s)',':'); hold off

pause

% Alternative, we can try Viterbi decoding to find the 'best' sequence of
% hidden states
[score,idx] = ghmm_vite( y, Pss, Ps0, Q, C, R );

% error is...
norm(s-idx,1)/T

% plot it, compared to 's'
subplot(311); hold on; h(3) = stairs(idx,'k'); 
legend(h,'s','soft', 'Viterbi'); hold off;

pause

% One can also estimate parameters of this HMM given the data
% sequence 'y'

% Enable printing of scores
global GHMM_VERBOSE GHMM_INFERENCE_MODE
GHMM_VERBOSE = 1;

% Initialize parameters using VQ
[Pss_0,Q_0,C_0,R_0,Ps0_0] = ghmm_init( y, 3, 2, 0, [], 'hlr');

% Now apply EM learning using exact inference
[Pss_1,Q_1,C_1,R_1,Ps0_1] = ghmm_learn(y, Pss_0,Q_0,C_0,R_0,Ps0_0, 1e-4);

printmat(Pss); printmat(Pss_0); printmat(Pss_1);
pause

printmat(Q); printmat(Q_0); printmat(Q_1);
pause

printmat(C); printmat(C_0); printmat(C_1);
pause

printmat(R); printmat(R_0); printmat(R_1);
pause

% Now apply EM learning using Viterbi inference
GHMM_INFERENCE_MODE = 'vite';
[Pss_1,Q_1,C_1,R_1,Ps0_1] = ghmm_learn(y, Pss_0,Q_0,C_0,R_0,Ps0_0, 1e-4);

printmat(Pss); printmat(Pss_0); printmat(Pss_1);
pause

printmat(Q); printmat(Q_0); printmat(Q_1);
pause

printmat(C); printmat(C_0); printmat(C_1);
pause

printmat(R); printmat(R_0); printmat(R_1);

pause

% Finally, here is how to combine simple hmms using probabilistic grammars

model{1}.name = 'entry state';
model{1}.Pss = [0 0 0;1 0 0; 0 1 0];
model{1}.Q = [ 0 1 0 ;
	       0 0 0 ;
	       0 0 0 ;
	       0 0 0 ;
	       0 0 0 ;
	       0 0 0 ;
	       0 0 0 ];
model{1}.C = -Inf*ones(1,7,3);
model{1}.R = zeros(1,7,3);

% This is HMM 1
model{2}.name = 'model 1';
model{2}.Pss = [ 0	0	0	0 ;
		 1	.9	0	0 ;
		 0	.1	.85	0 ;
		 0	0	.15	0 ];

model{2}.Q  = [ 0	0	0	0 ;
		0	.8	0	0 ;
		0	.1	.05	0 ;
		0	.05	.15	0 ;
		0	.05	.75	0 ;
		0	0	.05	0 ;
		0	0	0	0 ];
model{2}.C  = 10*rand(1,7,4);
model{2}.R  = rand(1,7,4);

% This is HMM 2
model{3}.name = 'model 2';
model{3}.Pss = [ 0	0	0	0 ;
		 1	.98	0	0 ;
		 0	.02	.95	0 ;
		 0	0	.05	0 ];

model{3}.Q  = [ 0	0	0	0 ;
		0	.1	0	0 ;
		0	.7	.05	0 ;
		0	.1	.25	0 ;
		0	.1	.65	0 ;
		0	0	.05	0 ;
		0	0	0	0 ];
model{3}.C  = 10*rand(1,7,4);
model{3}.R  = rand(1,7,4);

model{4}.name = 'exit state';
model{4}.Pss = [0 0 0;1 0 0; 0 1 0];
model{4}.Q = [ 0 0 0 ;
	       0 0 0 ;
	       0 0 0 ;
	       0 0 0 ;
	       0 0 0 ;
	       0 0 0 ;
	       0 1 0 ];
model{4}.C  = Inf*ones(1,7,3);
model{4}.R  = zeros(1,7,3);
% In gaussian hmms last state can be marked by Inf mean and zero variance

% This is the grammar
Pg = [ 0 0 0 0 ;
       1 .9 .1 0 ;
       0 .1 .8 0 ;
       0 0 .1 0 ];
% i.e., P(model(2) @ t | model(1) @ t-1 ) = .1, model 2 is followed by
% model one 10% of time

% Assuming that the model is discrete concatenate simple models into one
% complex model.
[Pss,Q,map,names] = dhmm_cat(Pg,model);

% Now generate a sequence using this model...
[y,s] = dhmm_sim(Pss,Q,[1 0 0 0 0]',inf);

% ...and decode it using the same model
[score,idx] = dhmm_vite( y, Pss, Ps0, Q  );

clf;
stairs([idx idx(end)]); axis tight;

if exist('mc_plotlab')

  % Decoded states
  mc_plotlab(idx,[],[1 1.5]);
  
  % Decoded models
  mc_plotlab(map(idx),names,[1 3.5]);
  
  % Original models
  mc_plotlab(map(s),names,[3.5 6],[0 1 0]);
  
end

pause

% or, if the model is Gaussian hmm, ...
[Pss,Q,C,R,map,names] = ghmm_cat(Pg,model);

% Now generate a sequence using this model...
[y,s] = ghmm_sim(Pss,Q,C,R,[1 0 0 0 0]',inf);

% ...and decode it using the same model
[score,idx] = ghmm_vite( y, Pss, Ps0, Q, C, R  );

clf;
stairs([idx idx(end)]); axis tight;

if exist('mc_plotlab')
  
  % Decoded states
  mc_plotlab(idx,[],[1 1.5]);
  
  % Decoded models
  mc_plotlab(map(idx),names,[1 3.5]);
  
  % Original models
  mc_plotlab(map(s),names,[3.5 6],[0 1 0]);
  
end
