function [cost,es,cs,scale] = hmm_fwd(Pss,Pys,Ps0,w)
%
% hmm_fwd -- HMM forward state estimation (forward probability propagation)
%
% Usage:
%   [cost,es,cs,scale] = hmm_fwd(Pss,Pys,Ps0,w)
% Input:
%   Pss		state transition probabilities [ to x from ]
%   Pys		evaluated observation LOG probabilities [ state x time ]
%   Ps0		initial state probabilities [ state x 1 ]
%   w		state transition probability weights (default 1)
% Output:
%   cost	-log Pr(Y_T) / T  
%   es		E[ s_t | Y_t ] = Pr( s_t | observation sequence up to time 't')
%   cs		E[ s_t s_{t-1}' | Y_t]
%   scale	log Pr( s_0, ..., s_t | observation sequence up to time 't')
%
% Note:
%   See notes for 'hmm_vite'.
%
% See also:
%   hmm, hmm_bwd, hmm_vite, hmm_smt, hmm_infr, hmm_sim
%

%	$Revision: 2.0 $ $Date: 2000/04/24 17:44:00 $

Pys = exp(Pys);

N = size(Pss,1);
T = size(Pys,2);
i = ones(1,N);

if (nargin<4)
  w = ones(1,T);
end

% Forward
es0 = Ps0.*Pys(:,1);
scale(1) = sum(es0);
es(:,1) = es0 / scale(1);

for t=2:T,
  es0 = ( ( Pss.^w(t) ) * es(:,t-1)).*Pys(:,t);
  scale(t) = sum(es0);
  es(:,t) = es0 / scale(t);
  if (nargout>2)
    cs(:,:,t-1) = ( Pss.^w(t) ) .* es(:,(t-1)*i)' .* Pys(:,t*i) / scale(t);
  end
end

scale = log(scale);
cost  = -sum(scale)/T;
