function [cost,es,cs] = hmm_infr(Pss,Pys,ps0,w)
%
% hmm_infr -- HMM inference
%
% Usage:
%   [cost,es,cs] = hmm_infr(Pss,Pys,ps0,w)
%
% Input:
%   Pss		state transition probabilities [ to x from ]
%   Pys		evaluated observation log probabilities [ state x time ]
%   Ps0		initial state probabilities [ state x 1 ]
%   w		state transition probability weights (default 1)
%
% Output:
%   cost	log Pr( observation sequence )
%   es		E[ s_t | observation sequence ]
%   cs		E[ s_t s_{t-1}' | observation sequence ]
%
% Global settings:
%   INFERENCE_MODE_HMM	'exact'	-- exact inference (default)
%			'vite'	-- Viterbi (one-wins-all) inference
%
% See also:
%   hmm_smt, hmm_vite, hmm_fwd, hmm_bwd, hmm_sim
%

%	$Revision: 2.0 $ $Date: 2000/04/24 17:44:01 $

global HMM_INFERENCE_MODE
if (~isempty(HMM_INFERENCE_MODE))
  inferMode = HMM_INFERENCE_MODE;
else
  inferMode = 'exact';
end

N = size(Pss,1);
T = size(Pys,2);

if (nargin<4)
  w = ones(1,T);
end

switch(lower(inferMode))
  
case 'exact',
  
  % Do regular inference
  
  switch (nargout),
  case 1,
    cost = hmm_smt(Pss,Pys,ps0,w);
  case 2,
    [cost,es] = hmm_smt(Pss,Pys,ps0,w);
  otherwise,
    [cost,es,cs] = hmm_smt(Pss,Pys,ps0,w);
  end
  
case 'vite',

  
  % Do Viterbi inference
  [cost,is] = hmm_vite(Pss,Pys,ps0,w);
  
  if (nargout>1)
    % Find es from is
    base = eye(N);
    es = base(:,is);
  end

  
  if (nargout>2)
  % Find cs from es
    for t=1:T-1,
      cs(:,:,t) = es(:,t+1)*es(:,t)';
    end
  end
    
 otherwise,
  
  error( sprintf('Unknown HMM_INFERENCE_MODE option %s.', upper(inferMode)) );
  
end


