function [cost,ex,vx,cx,err] = lds_fwd(y,A,C,Q,R,x0,Q0,u,w)
%
% lds_fwd -- linear dynamic system (lds) forward estimator
%
% Usage:
%   [cost,ex,vx,cx,err] = lds_fwd(Y,A,C,Q,R,x0,Q0,U,W)
% Input:
%   Y		observations
%   A		state transition matrix
%   C		observation matrix
%   Q		state noise covariance
%   R		observation noise covariance
%   x0		initial state mean
%   Q0		initial state noise covariance
%   U		deterministic system input
%   W		observation weights, R_t = R / W_t (0 for no observation)
% Output:
%   cost	-log{P(Y)} / T
%   ex		E[x_t|O_t]
%   vx		Var[x_t|O_t]
%   cx		Covar[x_t,x_(t-1)|O_t]
%   err		innovation error
%

% $Revision: 2.0 $ $Date: 2000/04/24 17:44:21 $

cost = 0;
[M,T] = size(y);

[M,N,L] = lds_dim(A,C,Q,R,x0,Q0);

if (nargin<8)
  u = [];
end

if (nargin<9)
  w = ones(1,T);
end

% Initialize
[a,c,q,r] = lds_param_fun(A,C,Q,R,1);

% Prioirs
ex_p = x0;
vx_p = Q0;

if isempty(u),
  [ex_P,vx_P,tmp,err(1)] = lds_f_lin( ex_p, vx_p, y(:,1), zeros(N,1), ...
				     eye(N,N*L), c, zeros(N), r/w(1) );
else
  [ex_P,vx_P,tmp,err(1)] = lds_f_lin( ex_p, vx_p, y(:,1), u(:,1), ...
				     eye(N,N*L), c, zeros(N), r/w(1) );
end

% Store posteriors
ex(:,1)	  = ex_P;
vx(:,:,1) = vx_P;

% Propagate
for t=2:T
   
  [a,c,q,r] = lds_param_fun(A,C,Q,R,t);
  
  % Priors
  ex_p = ex(:,t-1);
  vx_p = vx(:,:,t-1);
  
  % Posteriors
  if isempty(u),
    [ex_P,vx_P,cx_P,err(t)] = lds_f_lin( ex_p, vx_p, y(:,t), zeros(N,1), ...
					a, c, q, r/w(t) );
  else
    [ex_P,vx_P,cx_P,err(t)] = lds_f_lin( ex_p, vx_p, y(:,t), u(:,t), ...
					a, c, q, r/w(t) );
  end      
  
  % Store posteriors
  ex(:,t)	= ex_P;
  vx(:,:,t)	= vx_P;
  cx(:,:,t-1)	= cx_P;
  
end

cost = sum(err)/T + M*log(2*pi)/2;
