function [A,C,Q,R,x0] = lds_gen(type,varargin)
%
% lds_gen -- generate parameters for typical linear dynamic systems
%
% Usage:
%   [A,C,Q,R,x0] = lds_gen(model,params,...)
%
% Models:
%
%   (1) Newtonian dynamics
%
%       [A,C,Q,R,x0] = lds_gen('newton',N,T,Bc,Qc,Rc,x0c)
%       
%       N - dim(A)-1, e.g., N=1 is constant velocity model
%       T - sampling period
%	Bc - continuous-time input
%       Qc - continuous-time state noise variance
%	Rc - continuous-time measurement noise variance
%	x0c- continuous-time initial condition
%
%   (2) Continuous-to-discrete time model conversion
%
%       [A,C,Q] = lds_gen('gen',Ac,Bc,Cc,Qc,x0c,T)
%
%       Ac,Bc,Cc,Qc - continuous-time model parameters
%       T - sampling period
% 
% Note:
%   Both discretized models are augmented state models, i.e., constant
%   deterministic input is modeled by appending one (zero noise)
%   dimension to the state vector and expanding state transition matrix. 
%

% $Revision: 2.0 $ $Date: 2000/04/24 17:44:21 $

switch( lower(type) ),

case 'newton',
  
  N = varargin{1};
  T = varargin{2};
  b = varargin{3};
  q = varargin{4};
  r = varargin{5};
  x0= varargin{6};
  
  % Zero-order hold
  Ab  = expm( diag( ones(N+1,1), 1)*T );
  A = Ab(1:N,1:N);
  
  % Input on last component only
  B  = Ab(1:N,N+1)*b;

  % Combine A & B (augmented state model)
  A  = [ A B ; zeros(1,N) 1 ];
  
  % Factors of Q elements
  factQ(N) = 1;
  for d=N-1:-1:1,
    factQ(d) = factQ(d+1)*(N-d);
  end
  factQ = factQ'*factQ;
  
  % Exponents of Q elements
  powQ = log( exp(N-1:-1:0)'*exp(N-1:-1:0) ) + 1;
  
  Q = ( T.^powQ ) ./ (powQ.*factQ) * q;
  Q(N+1,N+1) = 0;

  % C
  C = [ 1 zeros(1,N) ];

  % R
  R = r;
  
  % x0
  x0 = [ x0; 1 ];
  
  
case 'gen',
  
 if ~exist('sym')
   error('Type gen requires symbolic math toolbox.');
 end
 
  Ac = varargin{1};
  Bc = varargin{2};
  Cc = varargin{3};
  Qc = varargin{4};
  T  = varargin{5};
  
  A = expm(Ac*T);
  
  t = sym('t','real');
  em = expm(Ac*(T-t));
  
  Bd = int( em*Bc, t, 0, T );
  for i=1:length(Bd),
    B(i,1) = eval(Bd(i));
  end
  
  % Combine A & B (augmented state model)
  A = [ A B; zeros(1,size(A,2)) 1 ];
  
  C = [ Cc zeros(size(C,1),1) ];
  
  Qd = int( em*Qc*em', 0, T );
  for i=1:size(Qd,1),
    for j=1:size(Qd,2),
      Q(i,j) = eval(Qd(i,j));
    end
  end
  
  Q(end+1,end+1) = 0;
  
otherwise
  
  error(sprintf('Type %s unknown.', type));
  
end
