function [A,C,Q,R,Q0,x0] = lds_init(y,N,mode,varargin)
%
% lds_init -- initialization of LDS parameters
%
% Usage:
%   [A,C,Q,R,Q0,x0] = lds_init(y,N,mode,...)
%
% Input:
%   y		observations
%   N		dimension of plant state space
%   mode	initialization mode:
%		'subspace' - subspace projection method
%		'fa' - factor analysis method
%
% Note:
%  [A,C,Q,R,Q0,x0] = lds_init(y,N,'subspace',ss_order)
%  [A,C,Q,R,Q0,x0] = lds_init(y,N,'fa',fa_error)
%

% $Revision: 2.0 $ $Date: 2000/04/24 17:44:22 $

Mode = { 'fa' 'subspace' };
ModeString = sprintf('\t%s\n',Mode{:});

if (~iscell(y))
  y = {y};
end

S = length(y);
M = size(y{1},1);


switch( lower(mode) )
  
 case Mode(1)
  
  if ~exist('fa','dir')
    error('No support for factor analysis');
  end
  
  % Put all data together and make it zero mean
  Y = [];
  for s = 1:S,
    
    T(s) = size(y{s},2);
    mY = mean(y{s},2);
    Y = [Y y{s} - mY(:,ones(1,T(s))) ];
    
  end
  
  if nargin<4,
    tol = 1e-3;
  else
    tol = varargin{1};
  end
  
  % Get observation parameters
  [C,R,ex] = lds_oinit(Y,N,M,S,tol);
  
  % Get plant parameters
  [A,Q,x0,Q0] = lds_pinit(ex,N,M,S,T);
  
 case Mode(2)
  
  % Subspace projection method. See L. Ljung, "System identification:
  % Theory for the user," 2nd edition, Prentice-Hall, 1999

  if nargin<4,
    r = 2*N;
  else
    r = varargin{1};
  end
  
  Y = []; Phi = [];
  
  for s = 1:length(y),
    
    T(s) = size(y{s},2);
    
    Yi = []; Phii = [];
    for i=1:r,
      Yi = [ Yi ; y{s}(:,r+i:end-r+i) ];
      Phii = [ y{s}(:,i:end-2*r+i) ; Phii ];
    end
    
    Y = [ Y  Yi ];
    Phi = [ Phi  Phii ];
  
  end
  
  % N4SID
  G = Y*Phi'*pinv(Phi*Phi')*Phi;
  
  [U,W,V] = svd(G);
  
  % Pick N largest components
  W = W(1:N,1:N);
  W1 = sqrtm(W);
  
  % Estimate observability matrix
  O = U(:,1:N)*W1;
  
  % Extract C & A from O
  C = O(1:M,:);
  A = O(1:M*(r-1),:) \ O(M+1:end,:);
  
  % Estimate X
  X = inv(W1)*W*V(:,1:N)';
    
  % Estimate Q & R
  Q = cov( ( X(:,2:end)-A*X(:,1:end-1) )' )'; Q = lim_var(Q);
  R = cov( ( Y(1:M,:) - C*X )' )'; R = lim_var(R);

  % Estimate x0 and Q0
  idx = cumsum([1 T]);
  X0 = X(:, idx(1:end-1) );
  if (S>1)
    x0 = mean( X0,2 );
    Q0 = cov( X0 );
  else
    x0 = X0;
    Q0 = eye(N);
  end
  Q0 = lim_var(Q0);
    
 otherwise
  
  error( sprintf('Unknown mode. Supported modes are:\n %s', ModeString) );
  
end
