function [a,c,q,r,x0,q0,cost,Ex] = lds_learn(Y,a,c,q,r,x0,q0,errThd,pY)
%
% lds_learn -- reestimation of LDS parameters
%
% Usage:
%   [a,c,q,r,x0,q0,cost] = lds_learn(y,a0,c0,q0,r0,x0,q00,eps,py)
% Input:
%   y		observations
%   a0...q00	initial parameter values
%   eps		accuracy
%   py		distribution of observations (default: uninformative)
% Output:
%   a...q0	reestimated parameter values
%   cost	cost history
% Global variables:
%   LS_VERBOSE		print information during iterations (default [])
%   LS_KMAX_LEARN	maximal number of iterations allowed (default 30)
%   LS_MAX_FUN		ls parameter maximization function (default lds_max)
% See also:
%   lds_init, ls
%

% $Revision: 2.0 $ $Date: 2000/04/24 17:44:22 $

global LS_VERBOSE LS_KMAX_LEARN LS_MAX_FUN

if (isempty(LS_KMAX_LEARN))
   kMax = 30;
else
   kMax = LS_KMAX_LEARN;
end

M = size(c,1);	% Observation size
N = size(a,1);	% State size

if (~iscell(Y))
   Y = { Y };
end

S = length(Y); % Number of training sequences

if nargin<9,
  pY = ones(1,S)/S;
end

cost = Inf;
err  = Inf;
k    = 0;
Ex   = {};

while (k < kMax & abs(err)>errThd),
  
  [Eyy,Eyx,Exx,ExxD,Ex0,Exx0,ExxT,Exx1,T] = lds_ics(N,M);
  
  costE = 0;
  
  for s=1:S,
    
    y = Y{s};
    Ts = size(y,2);

    % Expectation
    [costS,ex,vx,cx] = lds_infr(y,a,c,q,r,x0,q0);
    
    Ex{s} = ex;
    costE = costE + costS*Ts*pY(s);
    
    % Update statistics
    [Eyy,Eyx,Exx,ExxD,ExxT,Exx1,Ex0,Exx0,T] = ...
	lds_ucs( y, ex, vx, cx, Eyy,Eyx,Exx,ExxD,ExxT,Exx1,Ex0,Exx0,T,pY(s) );
    
  end
  
  
  % Cost before maximization
  costB = lds_cost(a,c,q,r,x0,q0,Exx,ExxD,Exx1,ExxT,Eyx,Eyy,Exx0,T);
  
  % Maximization
  if isempty(LS_MAX_FUN)
    [a,c,q,r,x0,q0] = lds_max( Eyy,Eyx,Exx,ExxD,ExxT,Exx1,Ex0,Exx0, T, ...
			      a,c,q,r,x0,q0 );
  else
    [a,c,q,r,x0,q0] = feval( LS_MAX_FUN, ...
			     Eyy,Eyx,Exx,ExxD,ExxT,Exx1,Ex0,Exx0, T, ...
			     a,c,q,r,x0,q0 );
  end
    
  % Cost after maximization
  costA = lds_cost(a,c,q,r,x0,q0,Exx,ExxD,Exx1,ExxT,Eyx,Eyy,Exx0,T);

  % Total cost after minimization
  cost(end+1) = (costE + costA - costB)/T;
  
  % Relative error
  err = ( cost(end-1) - cost(end) ) / abs( cost(end) );
  
  if ~isempty(LS_VERBOSE)
    fprintf('Iter (learning): %3d, Cost = %f, Err = %.4f%%\n', k, ...
	    cost(end), err*100);
  end
  
  k = k+1;
  
end

%%% end of lds_learn %%%
