/* $Revision: 2.1 $ $Date: 2000/06/01 14:54:34 $ */

#include <stdlib.h>
#include <stdio.h>
#include "../emf/math_util.h"
#include "lds_util.h"

#ifdef USE_MATLAB_MEMFUN
#define calloc	mxCalloc
#define free	mxFree
#define realloc	mxRealloc
#include <mex.h>
#endif

void
lds_clean()
{
  emf_clean();
}

void
mprint(double *A, int N, int M)
{
  int i,j;
  for (i=0;i<N;i++) {
    for (j=0;j<M;j++)
      printf("%f ", A[i+N*j]);
    printf("\n");
  }
}

#ifndef M_PI
#define M_PI		3.14159265358979323846
#endif


void
lds_gain( double *vxp, double *C, double *invO, int N, int M, 
	  double *K, double *Kv )
  /* K  = vxp*C'*invO */
  /* Kv = ( I - K*C ) */
{

  int i,j;
  
  prod( out_( vxp,C, N,N,M ),invO, N,M,M, K );
  
  if (Kv) {
    prod( K,C, N,M,N, Kv );
    for (i=0;i<N;i++)
      for (j=0;j<N;j++)
	Kv[i+j*N] = (i==j) - Kv[i+j*N];
  }

}

void
lds_predict_dT( double *ex, double *vx, double *u,
		double *A, double *Q,
		int N,
		double *exp, double *vxp, double *cxp )
{
  int i;
  double *tmp;

  /* exp = A*ex + U */
  if (u) 
    sum( prod_(A,ex,N,N,1), u, N, exp );
  else
    prod( A,ex,N,N,1,exp);

  /* cxp = A*vx */
  if (cxp) {
    prod( A,vx, N,N,N, cxp );
  }
  
  /* vxp = A*vx*A' + Q */
  if (vxp) {
    oqprod( A, vx, Q, N, N, vxp );
  }
}


double
lds_cost_dT( double *y, double *ex, double *vx, double *C, double *R, 
	     int N, int M )
{

  double detO, cost;
  double *dY;
  static double *invO = NULL;
  static int dimO = 0;
  
  invO = calloc(M*M,sizeof(double));
  
  /* invO = inv( C*vx*C' + R ) */
  sinv( oqprod_(C, vx, R, M, N), M, invO, &detO );	
  
  /* dY = y - C*ex  */
  dY = diff_( y, prod_(C,ex,M,N,1), M);
  
  /* Cost */
  cost = ( M*log(2*M_PI) + log(detO) + wnorm(invO,dY,M) )/2.;
  
  free(invO);

  return (cost);
}

/* Filtering */
static double *I_O_F, *O_F, *D_Y_F, *K_F, *KV_F, DET_O_F;

void
lds_filter_dT_alloc( int N, int M )
{
  I_O_F	= calloc(M*M, sizeof(double) );
  O_F	= calloc(M*M, sizeof(double) );
  D_Y_F	= calloc(M, sizeof(double) );
  K_F	= calloc(M*N, sizeof(double) );
  KV_F	= calloc(N*N, sizeof(double) );
}

void
lds_filter_dT_clean( int N, int M )
{
  int i;

  for(i=0;i<M*M;i++) I_O_F[i] = 0;
  for(i=0;i<M*M;i++) O_F[i] = 0;
  for(i=0;i<M;i++) D_Y_F[i] = 0;
  for(i=0;i<M*N;i++) K_F[i] = 0;
  for(i=0;i<N*N;i++) KV_F[i] = 0;
}

void
lds_filter_dT_free()
{
  int i;

  free( I_O_F );
  free( O_F );
  free( D_Y_F );
  free( K_F );
  free( KV_F );
}

double
lds_filter_dT( double *ex, double *vx, double *cx, double *y,
	       double *A, double *C, double *Q, double *R,
	       int N, int M,
	       double *exf, double *vxf, double *cxf )
{
  int i;
  double inovErr;

  /* Clean */
  lds_filter_dT_clean(N,M);

  /* Posterior */

  /* invO = inv( C*vxp*C' + R ) */
  sinv( oqprod_(C, vx, R, M, N), M, I_O_F, &DET_O_F );	
	
  /* dY = y - C*ex  */
  diff( y, prod_(C,ex,M,N,1), M, D_Y_F );
	
  /* Inovation error */
  inovErr = wnorm(I_O_F,D_Y_F,M);

  /* K = vxp*C'*invO */
  lds_gain( vx, C, I_O_F, N, M, K_F, KV_F );
      
  /* exf = ex + K*dY */
  sum( prod_(K_F,D_Y_F,N,M,1),ex, N, exf );
  
  /* vxf = (I - K*C)*vxp  */
  if (vxf)
    prod( KV_F,vx, N,N,N, vxf );
      
  /* cxf = (I - K*C)*cx */
  if (cxf)
    prod( KV_F,cx, N,N,N, cxf );
      
  /* Cost */
  return ( log(DET_O_F) + inovErr )/2.;

}

/* Smoothing */
static double *EXP_S1, *VXP_S1, *I_VXP_S1, *J_S1, *D_EXP_S1, *D_VXP_S1,
  *I_VXFP_S1, *D_VX_S1, DET_VXP_S1, DET_VXFP_S1;

void
lds_smooth_dT_alloc( int N )
{
  
  EXP_S1	= calloc( N, sizeof(double) );
  VXP_S1	= calloc( N*N, sizeof(double) );
  I_VXP_S1	= calloc( N*N, sizeof(double) );
  J_S1		= calloc( N*N, sizeof(double) );
  D_EXP_S1	= calloc( N, sizeof(double) );
  D_VXP_S1	= calloc( N*N, sizeof(double) );
  I_VXFP_S1	= calloc( N*N, sizeof(double) );
  D_VX_S1	= calloc( N*N, sizeof(double) );

}

void
lds_smooth_dT_free()
{
  free( EXP_S1 );
  free( VXP_S1 );
  free( I_VXP_S1 );
  free( J_S1 );
  free( D_EXP_S1 );
  free( D_VXP_S1 );
  free( I_VXFP_S1 );
  free( D_VX_S1 );

}

void
lds_smooth_dT_clean(int N )
{
  int i;

  for (i=0;i<N;i++) EXP_S1[i] = 0;
  for (i=0;i<N*N;i++) VXP_S1[i] = 0;
  for (i=0;i<N*N;i++) I_VXP_S1[i] = 0;
  for (i=0;i<N*N;i++) J_S1[i] = 0;
  for (i=0;i<N;i++) D_EXP_S1[i] = 0;
  for (i=0;i<N*N;i++) D_VXP_S1[i] = 0;
  for (i=0;i<N*N;i++) I_VXFP_S1[i] = 0;
  for (i=0;i<N*N;i++) D_VX_S1[i] = 0;
}

void
lds_smooth_dT( double *ex, double *vx, double *exf, double *vxf,
	       double *vxfp, double *cxf,
	       double *A, double *Q, double *u,
	       int N,
	       double *exN, double *vxN, double *cxN)
{
  /* 
     exP = A*exf;
     vxP = A*vxf*A' + Q;
     
     J = vxf*A'*pinv(vxP);
     
     exN = exf + J*(ex - exP);
     vxN = vxf + J*(vx - vxP)*J';
     cxN = cxf + (vx - vxfp)*sinv(vxfp)*cxf; 
  */
  
  lds_smooth_dT_clean(N);

  /* exP = A*exf + u */
  if (u)
    sum( prod_(A,exf,N,N,1), u, N, EXP_S1 );
  else
    prod(A,exf,N,N,1, EXP_S1);

  /* vxP = A*vxf*A' + Q */
  oqprod(A,vxf,Q,N,N, VXP_S1);

  /* ivxP = inv( vxP ) */
  sinv( VXP_S1, N, I_VXP_S1, &DET_VXP_S1 );

  /* J = vxf*A'*sinv(vxP) */
  prod( out_(vxf,A,N,N,N), I_VXP_S1, N,N,N, J_S1); 

  /* exN = exf + J*(ex - exP) */
  sum( exf, prod_(J_S1, diff_(ex,EXP_S1,N), N,N,1 ), N, exN );

  /* vxN = vxf + J*(vx - vxP)*J */
  oqprod( J_S1, diff_(vx,VXP_S1,N*N), vxf, N,N, vxN );

  /* cxN = cxf + (vx - vxfp)*sinv(vxfp)*cxf */
  sum( cxf, prod_( out_( diff_(vx,vxfp,N*N), sinv_(vxfp,N), N,N,N ), 
		   cxf, N,N,N), N*N, cxN ); 

}


static double *EXP_S2, *VXP_S2, *I_VXP_S2, *VXP1_S2, *I_VXP1_S2, *J_S2, *J1_S2,
  *J1_T_S2, *D_EXP_S2, *D_VXP_S2, *I_VXFP_S2, *D_VX_S2, *D_VXF_S2, *CX_S2,
  DET_VXP_S2, DET_VXP1_S2, DET_VXFP_S2;

void
lds_smooth2_dT_alloc( int N )
{
  
  EXP_S2	= calloc( N, sizeof(double) );
  VXP_S2	= calloc( N*N, sizeof(double) );
  I_VXP_S2	= calloc( N*N, sizeof(double) );
  VXP1_S2	= calloc( N*N, sizeof(double) );
  I_VXP1_S2	= calloc( N*N, sizeof(double) );
  J_S2		= calloc( N*N, sizeof(double) );
  J1_S2		= calloc( N*N, sizeof(double) );
  J1_T_S2	= calloc( N*N, sizeof(double) );
  D_EXP_S2	= calloc( N, sizeof(double) );
  D_VXP_S2	= calloc( N*N, sizeof(double) );
  I_VXFP_S2	= calloc( N*N, sizeof(double) );
  D_VX_S2	= calloc( N*N, sizeof(double) );
  D_VXF_S2	= calloc( N*N, sizeof(double) );
  CX_S2		= calloc( N*N, sizeof(double) );

}

void
lds_smooth2_dT_free()
{
  free( EXP_S2 );
  free( VXP_S2 );
  free( I_VXP_S2 );
  free( VXP1_S2 );
  free( I_VXP1_S2 );
  free( J_S2 );
  free( J1_S2 );
  free( J1_T_S2 );
  free( D_EXP_S2 );
  free( D_VXP_S2 );
  free( I_VXFP_S2 );
  free( D_VX_S2 );
  free( D_VXF_S2 );
  free( CX_S2 );
}

void
lds_smooth2_dT_clean(int N)
{
  int i;

  for (i=0;i<N;i++) EXP_S2[i] = 0;
  for (i=0;i<N*N;i++) VXP_S2[i] = 0;
  for (i=0;i<N*N;i++) I_VXP_S2[i] = 0;
  for (i=0;i<N*N;i++) VXP1_S2[i] = 0;
  for (i=0;i<N*N;i++) I_VXP1_S2[i] = 0;
  for (i=0;i<N*N;i++) J_S2[i] = 0;
  for (i=0;i<N*N;i++) J1_S2[i] = 0;
  for (i=0;i<N*N;i++) J1_T_S2[i] = 0;
  for (i=0;i<N;i++) D_EXP_S2[i] = 0;
  for (i=0;i<N*N;i++) D_VXP_S2[i] = 0;
  for (i=0;i<N*N;i++) I_VXFP_S2[i] = 0;
  for (i=0;i<N*N;i++) D_VX_S2[i] = 0;
  for (i=0;i<N*N;i++) D_VXF_S2[i] = 0;
  for (i=0;i<N*N;i++) CX_S2[i] = 0;
}

void
lds_smooth2_dT( double *ex, double *vx, double *cx, double *exf, double *vxf,
		double *vxfp,
		double *A, double *Q, double *Ap, double *Qp, double *u,
		int N,
		double *exN, double *vxN, double *cxN)
{
  /* 
     exP = A*exf;
     vxP = A*vxf*A' + Q;
     J   = vxf*A'*pinv(vxP);
     
     vxP1 = Ap*vxf1*Ap' + Qp;
     J1   = vxf1*Ap'*pinv(vxP1);
     
     exN = exf + J*(ex - exP);
     vxN = vxf + J*(vx - vxP)*J';
     cxN = vxf*J1' + J*(cx - A*vxf)*J1'; 
  */

  lds_smooth2_dT_clean(N);

  /* exP = A*exf + u */
  if (u)
    sum( prod_(A,exf,N,N,1), u, N, EXP_S2 );
  else
    prod(A,exf,N,N,1, EXP_S2);

  /* vxP = A*vxf*A' + Q */
  oqprod(A,vxf,Q,N,N, VXP_S2);

  /* ivxP = inv( vxP ) */
  sinv( VXP_S2, N, I_VXP_S2, &DET_VXP_S2 );

  /* J = vxf*A'*sinv(vxP) */
  prod( out_(vxf,A,N,N,N), I_VXP_S2, N,N,N, J_S2); 

  /* exN = exf + J*(ex - exP) */
  sum( exf, prod_(J_S2, diff_(ex,EXP_S2,N), N,N,1 ), N, exN );

  /* vxN = vxf + J*(vx - vxP)*J */
  oqprod( J_S2, diff_(vx,VXP_S2,N*N), vxf, N,N, vxN );

  if (cxN) {

    /* vxP1 = Ap*vxfp*Ap' + Qp */
    oqprod(Ap,vxfp,Qp,N,N, VXP1_S2);
    
    /* ivxP1 = inv( vxP1 ) */
    sinv( VXP1_S2, N, I_VXP1_S2, &DET_VXP1_S2 );
    
    /* J1 = vxfp*Ap'*sinv(vxP1) */
    prod( out_(vxfp,Ap,N,N,N), sinv_( VXP1_S2,N ), N,N,N, J1_S2 ); 
    
    /* cxN = vxf*J1' + J*(cx - A*vxf)*J1' */
    out( sum_( vxf, prod_( J_S2, diff_(cx, prod_(A,vxf,N,N,N),N*N), N,N,N),
	       N*N), J1_S2, N,N,N, cxN );
  }

}

void
lds_forward( double *A, double *C, double *Q, double *R, 
	     double *x0, double *Q0, double *U, double *W, double *Y,
	     int N, int M, int T,
	     double *ex, double *vx, double *cx , double *cost, double *err )
{
  
  int t;
  double cost_t, w, eps = 1e-6;
  double *Rw, *exp, *vxp, *cxp, *Rwt;

  /* "Allocate" routines */
  lds_filter_dT_alloc(N,M);

  /* Allocate intermediate prediction vars */
  exp = calloc(N,sizeof(double));
  vxp = calloc(N*N,sizeof(double));
  cxp = calloc(N*N,sizeof(double));

  /* Allocate scaled R */
  Rw = calloc(M*M,sizeof(double));

  /* Initialize */
  /* Scale factor */
  if (W) w = W[0]; else w = 1;

  if (w<eps) {
 
    /* Infinite measurement variance, do prediction only */
    if (U)
      sum( x0, U, N, ex );
    else
      copy( x0, N, ex );
    copy( Q0, N*N, vx );
    cost_t = 0;

  } else {

    /* Scale R if needed */
    if ( fabs(1-w) > eps ) {
      scale(R,1/w,M*M,Rw);
      Rwt = Rw;
    } else
      Rwt = R;
  
    /* Predict & filter */
    cost_t = lds_filter_dT( x0, Q0, NULL, Y, A, C, Q, Rwt, N, M, 
			    ex, vx, NULL );
    
  }

  /* Cost */
  *cost = cost_t;
  if (err)
    err[0] = cost_t;


  /* Propagate */
  for(t=1;t<T;t++) {

    
    /* Update pointers */
    Y  += M;
    if ( U ) U += N;
    ex += N;
    vx += N*N;
    if (t>1) cx += N*N;

    /* Measurement scale */
    if (W) w = W[t]; else w = 1;
    
    if (w<eps) {
      
      /* Infinite measurement variance, do prediction only */
      lds_predict_dT( ex-N, vx-N*N, U, A, Q, N, ex, vx, cx );
      cost_t = 0;
      
    } else {
      
      /* Scale R if needed */
      if ( fabs(1-w) > eps ) {
	scale(R,1/w,M*M,Rw);
	Rwt = Rw;
      } else
	Rwt = R;
  
      /* Predict & filter */
      lds_predict_dT( ex-N, vx-N*N, U, A, Q, N, exp, vxp, cxp );
      cost_t = lds_filter_dT( exp, vxp, cxp, Y, A, C, Q, Rwt, N, M, 
			      ex, vx, cx );

    }

    /* Cost */
    *cost += cost_t;
    if (err)
      err[t] = cost_t;

  } /* t=1:T-1 */

  /* Normalized cost per time step */
  *cost = *cost/T + M*log(2.*M_PI)/2.;

  /* Free routines */
  lds_filter_dT_free();

  free(exp); free(vxp); free(cxp);
  free(Rw);

}

void
lds_tv_forward( sysFun A_fun, sysFun C_fun, sysFun Q_fun, sysFun R_fun, 
		double *x0, double *Q0, double *U, double *W, double *Y,
		int N, int M, int T,
		double *ex, double *vx, double *cx , 
		double *cost, double *err )
{
  
  int t;
  double cost_t, w, eps = 1e-6;
  double *Rw, *exp, *vxp, *cxp, *Rwt;
  double *At, *Ct, *Qt, *Rt;

  /* "Allocate" routines */
  lds_filter_dT_alloc(N,M);

  /* Allocate intermediate prediction vars */
  exp = calloc(N,sizeof(double));
  vxp = calloc(N*N,sizeof(double));
  cxp = calloc(N*N,sizeof(double));

  /* Allocate scaled R */
  Rw = calloc(M*M,sizeof(double));

  /* Initialize */

  /* Parameters */
  At = A_fun(0,x0); Ct = C_fun(0,x0); 
  Qt = Q_fun(0,x0); Rt = R_fun(0,x0);

  /* Scale factor */
  if (W) w = W[0]; else w = 1;

  if (w<eps) {
 
    /* Infinite measurement variance, do prediction only */
    if (U)
      sum( ex, U, N, ex );
    else
      copy( x0, N, ex );
    copy( Q0, N*N, vx );
    cost_t = 0;

  } else {

    /* Scale R if needed */
    if ( fabs(1-w) > eps ) {
      scale(Rt,1/w,M*M,Rw);
      Rwt = Rw;
    } else
      Rwt = Rt;
  
    /* Predict & filter */
    cost_t = lds_filter_dT( x0, Q0, NULL, Y, At, Ct, Qt, Rwt, N,M, ex, vx, NULL );

  }

  /* Cost */
  *cost = cost_t;
  if (err)
    err[0] = cost_t;


  /* Propagate */
  for(t=1;t<T;t++) {

    
    /* Update pointers */
    Y  += M;
    if ( U ) U += N;
    ex += N;
    vx += N*N;
    if (t>1) cx += N*N;

    /* Parameters */
    At = A_fun(t,ex-N); Ct = C_fun(t,ex-N); 
    Qt = Q_fun(t,ex-N); Rt = R_fun(t,ex-N);
    
    /* Measurement scale */
    if (W) w = W[t]; else w = 1;
    
    if (w<eps) {
      
      /* Infinite measurement variance, do prediction only */
      lds_predict_dT( ex-N, vx-N*N, U, At, Qt, N, ex, vx, cx );
      cost_t = 0;
      
    } else {
      
      /* Scale R if needed */
      if ( fabs(1-w) > eps ) {
	scale(Rt,1/w,M*M,Rwt);
	Rwt = Rw;
      } else
	Rwt = Rt;

  
      /* Predict & filter */
      lds_predict_dT( ex-N, vx-N*N, U, At, Qt, N, exp, vxp, cxp );
      cost_t = lds_filter_dT( exp, vxp, cxp, Y, At, Ct, Qt, Rwt, N,M, ex, vx, cx );

    }

    /* Cost */
    *cost += cost_t;
    if (err)
      err[t] = cost_t;

  } /* t=1:T-1 */

  /* Normalized cost per time step */
  *cost = *cost/T + M*log(2*M_PI)/2;

  /* Free routines */
  lds_filter_dT_free();

  free(exp); free(vxp); free(cxp);
  free(Rw);

}

void
lds_smooth( double *A, double *Q, double *u,
	double *exf, double *vxf, double *cxf,
	int N, int T,
	double *ex, double *vx, double *cx )
{

  int t;

  /* Allocate routine */
  lds_smooth_dT_alloc(N);

  /* Initialize */
  ex += N*(T-1); vx += N*N*(T-1); cx += N*N*(T-1);
  exf += N*(T-1); vxf += N*N*(T-1); cxf += N*N*(T-1);
  if (u) u += N*(T-1);

  copy( exf, N, ex );
  copy( vxf, N*N, vx );

  /* Propagate */
  for (t=T-2;t>=0;t--) {

    /* Update pointers */
    ex -= N;  vx -= N*N; cx -= N*N;
    exf -= N; vxf -= N*N; cxf -= N*N;
    if (u) u -= N;

    /* Smooth */
    lds_smooth_dT( ex+N, vx+N*N, exf, vxf, vxf+N*N, cxf, A, Q, u, N, ex, vx, cx);

  }

  /* Free routine */
  lds_smooth_dT_free();
    
    
}

void
lds_tv_smooth( sysFun A_fun, sysFun Q_fun, double *u,
	   double *exf, double *vxf, double *cxf,
	   int N, int T,
	   double *ex, double *vx, double *cx )
{

  int t;
  double *At, *Qt;

  /* Allocate routine */
  lds_smooth_dT_alloc(N);

  /* Initialize */
  ex += N*(T-1); vx += N*N*(T-1); cx += N*N*(T-1);
  exf += N*(T-1); vxf += N*N*(T-1); cxf += N*N*(T-1);
  if (u) u += N*(T-1);

  copy( exf, N, ex );
  copy( vxf, N*N, vx );

  /* Propagate */
  for (t=T-2;t>=0;t--) {

    /* Parameters */
    At = A_fun(t+1,ex+N); Qt = Q_fun(t+1,ex+N);

    /* Update pointers */
    ex -= N;  vx -= N*N; cx -= N*N;
    exf -= N; vxf -= N*N; cxf -= N*N;
    if (u) u -= N;

    /* Smooth */
    lds_smooth_dT( ex+N, vx+N*N, exf, vxf, vxf+N*N, cxf, At, Qt, u, N, ex, vx, cx);

  }

  /* Free routine */
  lds_smooth_dT_free();
    
    
}

void
lds_smooth2( double *A, double *Q, double *C, double *R, double *U,
	 double *exf, double *vxf,
	 int N, int M, int T,
	 double *ex, double *vx, double *cx )
{

  double *K, *Kv, *vxp;
  int t;

  /* Allocate routine etc */
  lds_smooth2_dT_alloc(N);
  K = calloc(N*M,sizeof(double)); Kv = calloc(N*N,sizeof(double)); 
  vxp = calloc(N*N,sizeof(double));

  /* Initialize */
  ex += N*(T-1); vx += N*N*(T-1); cx += N*N*(T-2);
  exf += N*(T-1); vxf += N*N*(T-1);
  if (U) U += N*(T-1);

  copy( exf, N, ex );
  copy( vxf, N*N, vx );
  
  /* vxp = A*vx*A' + Q */
  /* K = vxp*C'*inv(C*vxp*C'+R) */
  /* Kv = I - C*K */
  /* cx = Kv*A*vx */
  oqprod(A,vx-N*N,Q, N,N, vxp);
  lds_gain( vxp, C, sinv_( oqprod_(C,vxp,R,N,N), N), N, M, K, Kv );
  out( prod_(Kv,A,N,N,N), vx-N*N, N,N,N, cx );

  /* Propagate */
  for (t=T-2;t>0;t--) {

    /* Update pointers */
    ex -= N;  vx -= N*N; cx -= N*N;
    exf -= N; vxf -= N*N;
    if (U) U -= N;

    /* Smooth */
    lds_smooth2_dT( ex+N, vx+N*N, cx+N*N, exf, vxf, vxf+N*N, A, Q, A, Q, U, N,
		ex, vx, cx);

  }

  /* Finish */

  /* Update pointers */
  ex -= N;  vx -= N*N;
  exf -= N; vxf -= N*N;
  if (U) U -= N;
  
  /* Smooth */
  lds_smooth2_dT( ex+N, vx+N*N, cx+N*N, exf, vxf, vxf+N*N, A, Q, A, Q, U, N,
	      ex, vx, NULL);
  
  

  /* Free routine */
  lds_smooth2_dT_free();
  free(K);free(Kv);free(vxp);
    
}


void
lds_tv_smooth2( sysFun A_fun, sysFun Q_fun, sysFun C_fun, sysFun R_fun,
	    double *U,
	    double *exf, double *vxf,
	    int N, int M, int T,
	    double *ex, double *vx, double *cx )
{

  double *At, *Qt, *Atp, *Qtp, *Ctp, *Rtp;
  double *K, *Kv, *vxp;
  int t;

  /* Allocate routine etc */
  lds_smooth2_dT_alloc(N);
  K = calloc(N*M,sizeof(double)); Kv = calloc(N*N,sizeof(double)); 
  vxp = calloc(N*N,sizeof(double));

  /* Initialize */
  ex += N*(T-1); vx += N*N*(T-1); cx += N*N*(T-2);
  exf += N*(T-1); vxf += N*N*(T-1);
  if (U) U += N*(T-1);

  copy( exf, N, ex );
  copy( vxf, N*N, vx );

  /* Parameters */
  Atp = A_fun(T-1,ex); Qtp = Q_fun(T-1,ex); 
  Ctp = C_fun(T-1,ex); Rtp = R_fun(T-1,ex);
  
  /* vxp = A*vx*A' + Q */
  /* K = vxp*C'*inv(C*vxp*C'+R) */
  /* Kv = I - C*K */
  /* cx = Kv*A*vx */
  oqprod(Atp,vx-N*N,Qtp, N,N, vxp);
  lds_gain( vxp, Ctp, sinv_( oqprod_(Ctp,vxp,Rtp,N,N), N), N, M, K, Kv );
  out( prod_(Kv,Atp,N,N,N), vx-N*N, N,N,N, cx );

  /* Propagate */
  for (t=T-2;t>0;t--) {

    /* Parameters */
    At = Atp; /* A_fun(t+1,ex); */
    Qt = Qtp; /* Qt = Q_fun(t+1,ex); */ 
    Atp = A_fun(t,ex);
    Qtp = Q_fun(t,ex);

    /* Update pointers */
    ex -= N;  vx -= N*N; cx -= N*N;
    exf -= N; vxf -= N*N;
    if (U) U -= N;

    /* Smooth */
    lds_smooth2_dT( ex+N, vx+N*N, cx+N*N, exf, vxf, vxf+N*N, At, Qt, Atp, Qtp, 
		U, N, ex, vx, cx);

  }

  /* Finish */

  /* Parameters */
  At = Atp; /* A_fun(t+1,ex); */
  Qt = Qtp; /* Qt = Q_fun(t+1,ex); */ 
  Atp = A_fun(t,ex);
  Qtp = Q_fun(t,ex);
  
  /* Update pointers */
  ex -= N;  vx -= N*N;
  exf -= N; vxf -= N*N;
  if (U) U -= N;
  
  /* Smooth */
  lds_smooth2_dT( ex+N, vx+N*N, cx+N*N, exf, vxf, vxf+N*N, At, Qt, Atp, Qtp, U, N,
	      ex, vx, NULL);
  
  

  /* Free routine */
  lds_smooth2_dT_free();
  free(K);free(Kv);free(vxp);
    
}


double
lds_cost_state_dT( double *A, double *Q, double *ex, double *vx, double *exp, 
		   double *vxp, double *cx, int N )
{
  double cost, *invQ, detQ;
  int N2;

  N2 = N*N;
  invQ = calloc(N2,sizeof(double));

  sinv( Q, N, invQ, &detQ);

  /* cost = log( det(Q) ) + tr{ inv(Q)*vx - 2*A'inv(Q)*cx + A'inv(Q)A*vxp } */
  cost = ( 
	  log( detQ )
	  + trace( prod_(invQ, sum_(vx, out_(ex,ex,N,1,N), N2), N,N,N), N )
	  - 2*trace( prod_( dot_(A,invQ,N,N,N), 
			    sum_(cx, out_(ex,exp,N,1,N), N2), N,N,N), N )
	  + trace( prod_( iqprod_(A,invQ,NULL,N,N), 
			  sum_(vxp, out_(exp,exp,N,1,N), N2), N,N,N), N ) 
	  ) / 2.;

  free(invQ);

  return(cost);

}

double
lds_cost_state_0( double *x0, double *Q0, double *ex, double *vx, int N )
{
  double cost, *invQ0, detQ0;
  int N2;

  N2 = N*N;
  invQ0 = calloc(N2,sizeof(double));

  sinv( Q0, N, invQ0, &detQ0);

  /* cost = log( det(Q) ) + 
     tr{ inv(Q)*(vx+ex*ex') } - 2*x0'inv(Q)*ex + x0'inv(Q)x0 */
  cost = ( 
	  log( detQ0 ) 
	  + trace( prod_(invQ0, sum_(vx, out_(ex,ex,N,1,N), N2), N,N,N), N )
	  - (*dot_(x0, prod_(invQ0,ex,N,N,1), 1,N,1))*2.
	  + *iqprod_(x0,invQ0,NULL,1,N) 
	  ) / 2.;

  free(invQ0);

  return(cost);
}


