function [P,map,names] = mc_cat( Psyntax, modelList )
%
% mc_comb -- combine simple MCs into one complex MC
%
% Usage:
%   [Pss,map,names] = mc_comb( Psyntax, modelList )
%
% Input:
%   Psyntax	model transition probabilities
%   modelList	list of models & parameter
%
% Output:
%   Pss		complex model's state transition probability matrix
%   map		model state space mapping
%   names	list of simple model names contained in complex model
%
% Note:
%   (1) First state of each model is its entry state and last state is the
%       exit state.  Both states are 'non-visiting', i.e., expected durations
%       of those states are zero.  Hence, first columns of every Pss can
%       be viewed as initial state distributions and last columns are always
%       zero.
%   (2) Model transition probability Psyntax(i,j) is the probability that
%       the exit state of model 'j' transitions into the entry state of
%       model 'i'.
%   (3) Each element of model list can take one of two forms:
%		{ name, Pss } or 
%		structure with 'Pss' and 'name' fields.
%       Pss is the simple model's parameters and 'name' is the
%       model's name.
%   (4) Mapping 'map' maps states of the complex model to simple models,
%       i.e., names{ map(2,j) } is the name of the simple model whose state
%       was mapped to state 'j' of the complex model.
%
%

%	$Revision: 2.0 $ $Date: 2000/04/24 17:43:07 $


P = []; pO = []; pI = []; map = []; cIdx = 0; names = {};
L = length(modelList);

fact = sum(Psyntax-diag(diag(Psyntax)));

for i=1:L,
  
  pR = [];
  
  if (isstruct(modelList{i}))
    Pss_i = modelList{i}.Pss;
  else
    Pss_i = modelList{i}{1};
  end
  [N,M] = size(Pss_i);
  
  map = [ map  ones(1,N-2)*i ];
  
  if (isstruct(modelList{i}))
    names = { names{1:end} modelList{i}.name };
  else
    names = { names{1:end} modelList{i}{2} };
  end
  cIdx = cIdx + N-2;
  
  Pss_R_i = Pss_i(2:N-1,2:M-1);
  pi_i  = Pss_i(2:N-1,1);
  po_i  = Pss_i(N,2:M-1);
  
  for j=1:L,
    
    if (isstruct(modelList{i}))
      Pss_j = modelList{j}.Pss;
    else
      Pss_j = modelList{j}{1};
    end
    [N,M] = size(Pss_j);
    
    PssR_j = Pss_j(2:N-1,2:M-1);
    pi_j   = Pss_j(2:N-1,1);
    po_j   = Pss_j(N,2:M-1);
    
    if (j==i)
      pR = [ pR ; PssR_j ];
    else
      if fact(i)
	pR = [ pR ; Psyntax(j,i)/fact(i)*pi_j*po_i ];
      else
	pR = [ pR ; Psyntax(j,i)*0*pi_j*po_i ];
      end
    end
    
  end
  
  P = [P  pR];
  
end

[N,M] = size(P);
