function [es,cs,is] = mc_gen(N,T,mode,varargin)
%
% mc_gen -- generate default Markov chain sequences
%
% Usage:
%   [es,cs,is] = mc_gen(N,T,mode,param)
% Input:
%   N		number of states
%   T		duration
%   mode	'hlr' - left-to-right with hard transitions
%		'slr' - left-to-right with soft transitions
%		        (softness in 'param') 
%		'lt'  - lower triangular P
%		'full'- full P
%               'diag'- specify average state lengths in 'param'
% Output:
%   es		E[ s_t ] = Pr( s_t )
%   cs		E[ s_t s_{t-1}' ] = Pr ( s_t s_{t-1}' )
%   is		decoded states
%

%	$Revision: 2.0 $ $Date: 2000/04/24 17:43:08 $

global MC_HTK_MODE
if (isempty(MC_HTK_MODE))
  htkMode = 0;
else
  htkMode = 1;
end

if (htkMode)
  N = N-2;
end

switch (lower(mode))
case 'hlr',
  
  % "Hard" left-to-right 
  for n=1:N,
    iB = round(T/N*(n-1)+1);
    iE = round(T/N*n);
    es(n,iB:iE) = 1;
    is(iB:iE) = n;
  end
  
case 'hlrc',
  
  % "Hard" left-to-right cyclic
  for n=1:N,
    iB = round((T-1)/N*(n-1)+1);
    iE = round((T-1)/N*n);
    es(n,iB:iE) = 1;
    is(iB:iE) = n;
  end
  es(1,T) = 1;
  is(T) = 1;
  
case 'slr',
  
  % "Soft" left-to-right 
  t = 1:T;
  for n=1:N,
    
    iB = round(T/N*(n-1)+1);
    eB = 1./( 1 + exp( -(t-iB)/varargin{1}(1) ) );
    iE = round(T/N*n);
    eE = 1./( 1 + exp( (t-iE)/varargin{1}(1) ) );
    
    if (n==1)
      es(n,:) = eE;
    elseif (n==N)
      es(n,:) = eB;
    else
      es(n,:) = eE + eB - 1;
    end
    
    is(iB:iE) = n;
    
  end
  ses = sum(es,1);
  es = es./ses(ones(N,1),:);

case 'lt',
  
  % Lower triangular
  for n=1:N,
    iB = round(T/N*(n-1)+1);
    iE = T;
    es(n:N,iB:iE) = 1;
  end
  ses = sum(es,1);
  es = es./ses(ones(N,1),:);

case 'full',
  
  % Full
  es = ones(N,T)/N;
  is = ceil( rand(1,T)*N );
  
  
case 'diag',
  
  es = []; is = [];
  N = length(varargin{1});
  for k=1:N,
    is = [is ones(1,varargin{1}(k))*k];
  end
  base = eye(N);
  es = base(:,is);
  T  = length(is);

case 'cyc',
  
  es = []; is = []; esP = [];
  N  = length(varargin{1});
  base = eye(N);

  p0 = base(:,varargin{2});

  k1 = 1;
  for k = [varargin{2}:N 1:varargin{2}-1],
    isP = [isP ones(1,varargin{1}(k1))*k];
    k1 = k1+1;
  end
  
  while(length(is)<T)
    is = [is isP];
  end
  is = is(:,1:T);

  es = base(:,is);
  
end

if (htkMode)
  es = [zeros(1,T);es;zeros(1,T)];
  is = is + 1;
end

if (nargout>1)
  % Find cs from es
  for t=1:T-1,
    cs(:,:,t) = es(:,t+1)*es(:,t)';
  end
end
