function [P,p0,cost] = mc_learn(y,P,p0,errThd,w)
%
% mc_learn -- learn Markov chain parameters
%
% Usage:
%   [P,p0,cost] = mc_learn(y,P,p0,err,w)
%
% Input:
%   y		list of observation sequences
%   P		initial state transition probability matrix
%   p0		initial state probabilities
%   err		max. relative error
%   w		distribution of observation sequences (default uninformative)
%
% Output:
%   P		reestimated state transition probability matrix
%   p0		reestimated initial state distribution
%   cost	cost history
%

%	$Revision: 2.0 $ $Date: 2000/04/24 17:43:09 $

global MC_VERBOSE MC_KMAX_LEARN MC_MAX_FUN

if (~iscell(y))
  y = {y};
end

N = size(P,2);
S = length(y);

basis = eye(N);

if nargin < 4,
  errThd = 1e-3;
end

if nargin < 5,
  w = ones(1,S)/S;
end

if (~isempty(MC_KMAX_LEARN))
   kMax = MC_KMAX_LEARN;
else
   kMax = 30;
end

k    = 0;
err  = realmax;
cost = realmax;

while(k < kMax & abs(err)>errThd)


  % Initialize statistics
  [Ess,Es0,T] = mc_ics(N);

  costE = 0;
  for s = 1:S,
    
    Ts = size(y{s},2);
    
    % Inference
    [costEs,es,cs] = mc_infr(y{s}, lin2log(P), lin2log(p0) );
  
    costE = costE + costEs*w(s)*Ts;
    
    % Update stats
    [Ess,Es0,T] = mc_ucs(es,cs,Ess,Es0,T,w(s));
      
  end

  % Cost before maximization
  costB = mc_cost( Ess, Es0, P, p0, T);
  
  % Maximize
  if isempty(MC_MAX_FUN)
    [P,p0] = mc_max( Ess, Es0, T, P, p0 );  
  else
    [P,p0] = feval( MC_MAX_FUN, Ess,Es0,T,P,p0 );
  end
    
  % Cost after maximization
  costA = mc_cost( Ess, Es0, P, p0, T);
  
  % Actual cost
  cost(end+1) = (costE + costA - costB)/T;
  
  % Relative error
  err = ( cost(end-1) - cost(end) ) / abs( cost(end) );

  if (~isempty(MC_VERBOSE))
    fprintf('Iter (learning): %3d, Cost = %f, Err = %.4f%%\n', k, ...
            cost(end), err*100);
  end
   
  k = k + 1;
  
  
end % while
