function [gQ,gC,gRI] = mog_grad(y,Q,C,R)
%
% mog_grad -- gradient of Gaussian mixture (MOG) entropy cost
%
% Gradient in the parameter space of the MOG model of the cost function 
% E [ log mog(x,q,c,r) ], evaluated at (Q,C,R) for a set of data points 'y', 
% where 'E' is the expectation operator.
%
% Usage:
%   [gQ,gC,gRI] = mog_grad(y,Q,C,R)
% Input:
%   y	data set
%   Q	log mixture weights
%   C	mixture means
%   R	mixture variances (diagonal or full)
% Output:
%   gQ	gradient of mixture weights
%   gC	gradient of mixture means
%   gRI	gradient of the inverse of mixture variance
%
% See also:
%   mog, mog_smt
%

%	$Revision: 1.5 $ $Date: 2000-03-03 14:26:00-05 $

T = size(y,2);
M = size(Q,1);
N = size(C,1);
S = 1;

% Inference
[cost,ecs] = mog_smt(y,Q,C,R);

gQ = zeros(M,S);
gC = zeros(N,M,S);
gRI = zeros(N,M,S);

P = exp(P);
Q = exp(Q);

% Gradient
for s=1:S,
  for m=1:M,
  
    Cms = C(:,m,s);
    Rms = R(:,m,s);
    
    diff =  y - Cms*ones(1,T);
    Ecs  =  ones(N,1)*squeeze( ecs(m,s,:) );
    % Mean
    gC(:,m,s) = diag(1./Rms) * sum( diff .* Ecs, 2 ) / T;

    % Inverse variance
    gRI(:,m,s) = ( Rms*sum(ecs(m,s,:)) - sum( diff.^2 .* Ecs, 2 ) ) / (2*T);

    % Mixture coefficient
    gQ(m,s) = sum(ecs(m,s,:))/Q(m,s)/T;
  
  end
    
end

% Projection of gQ onto constraint surface ( surf. normal ones(M,1)/sqrt(M) )
gQ = gQ - ones(M,1)*sum(gQ,1)/M;

