function [Q,C,R,cost] = mog_learn(y,Q,C,R,errThd,w)
%
% mog_learn -- learn mixture of Gaussians parameters
%
% Usage:
%   [Q,C,R,cost] = mog_learn(y,Q,C,R,err,w)
%
% Input:
%   y		list of observation sequences
%   Q		initial mixture weights
%   C		initial observation means
%   R		initial observation variances
%   err		max. relative error
%   w		distribution of observation sequences (default uninformative)
%
% Output:
%   Q		updated mixture weights
%   C		updated observation means
%   R		updated observation variances
%   cost	cost history
%

%	$Revision: 2.0 $ $Date: 2000/04/24 17:42:14 $



global MOG_VERBOSE MOG_KMAX_LEARN MOG_MAX_FUN

M = length(Q);
D = size(C,1);
d = ndims(R)-ndims(C);

S = length(y);

if nargin < 5,
  errThd = 1e-3;
end

if nargin < 6,
  w = ones(1,S)/S;
end

if (~isempty(MOG_KMAX_LEARN))
   kMax = MOG_KMAX_LEARN;
else
   kMax = 30;
end

k    = 0;
err  = realmax;
cost = realmax;

while(k < kMax & abs(err)>errThd)


  % Initialize statistics
  [Ec,Eyc,Eyyc,T] = mog_ics(M,D,d);

  % Inference
  [costEs,ec] = mog_infr(y, Q,C,R );
  costE = costEs*w';
  
  % Update stats
  [Ec,Eyc,Eyyc,T] = mog_ucs(y,ec, Ec,Eyc,Eyyc,T, w);
  
  % Cost before maximization
  costB = mog_cost( Ec, Eyc, Eyyc, Q, C, R, T);
  
  % Maximize
  if isempty(MOG_MAX_FUN)
    [Q,C,R] = mog_max( Ec, Eyc, Eyyc, Q, C, R, T );  
  else
    [Q,C,R] = feval( MOG_MAX_FUN, Ec, Eyc, Eyyc, Q, C, R, T );
  end
    
  % Cost after maximization
  costA = mog_cost( Ec, Eyc, Eyyc, Q, C, R, T);
  
  % Actual cost
  cost(end+1) = (costE + costA - costB)/T;
  
  % Relative error
  err = ( cost(end-1) - cost(end) ) / abs( cost(end) );

  if (~isempty(MOG_VERBOSE))
    fprintf('Iter (learning): %3d, Cost = %f, Err = %.4f%%\n', k, ...
            cost(end), err*100);
  end
   
  k = k + 1;
  
  
end % while
